import typer

from typing_extensions import Annotated
from pathlib import Path
from rich import print as rprint

from ow.mcp.install import install


def register_mcp_install(app: typer.Typer):
    @app.command(name="install")
    def mcp_install(
        client: Annotated[
            str,
            typer.Option("--client", help="Target client to install for."),
        ] = "claude-code",
        include_agent: Annotated[bool, typer.Option("--include-agent")] = True,
        project_path: Annotated[str | None, typer.Option("--project-path")] = None,
    ) -> None:
        import ow

        # Determine project root path
        if project_path:
            ow_path = Path(project_path)
        else:
            # Path(ow.__file__) example:
            # /GitHub/additive-manufacturing-agent/.venv/lib/python3.13/site-packages/ow
            # Going up 5 levels to get to the project root
            ow_path = Path(ow.__file__).parents[5]

        rprint(
            f"[bold green]Using `out-workspace` packaged under project path:[/bold green] {ow_path}"
        )

        install(ow_path, client=client, include_agent=include_agent)

    _ = app.command(name="install")(mcp_install)
    return mcp_install



import sys
import typer

from rich.console import Console
from rich import print as rprint

app = typer.Typer(
    name="out-workspace",
    help="Manage workspace folders under `out/`",
    add_completion=False,
    no_args_is_help=True,
)


def _rich_exception_handler(exc_type, exc_value, exc_traceback):
    """Handle exceptions with rich formatting."""
    if exc_type is KeyboardInterrupt:
        rprint("\n ⚠️  [yellow]Operation cancelled by user[/yellow]")
        sys.exit(1)
    else:
        sys.__excepthook__(exc_type, exc_value, exc_traceback)


sys.__excepthook__ = _rich_exception_handler

console = Console()



import os
import subprocess
import sys
import typer
from importlib.resources import files

from pathlib import Path
from rich import print as rprint


def register_mcp_development(app: typer.Typer):
    @app.command(name="development")
    def mcp_development() -> None:
        from mcp.cli import cli

        cwd = Path.cwd()

        # try:
        rprint(f"Starting MCP Development Server")

        # Get the correct npx command
        npx_cmd = cli._get_npx_command()

        if not npx_cmd:
            cli.logger.error(
                "npx not found. Please ensure Node.js and npm are properly installed and added to your system PATH."
            )
            raise typer.Exit(1)

        # Run the MCP Inspector command with shell=True on Windows
        shell = sys.platform == "win32"

        file_spec = files("ow.mcp").joinpath("__main__.py")

        print(file_spec)

        uv_cmd = ["uv", "run", "--with", "mcp", "mcp", "run", str(file_spec)]
        print(f"uv_cmd: {uv_cmd}")

        process = subprocess.run(
            [npx_cmd, "@modelcontextprotocol/inspector"] + uv_cmd,
            check=True,
            shell=shell,
            env=dict(os.environ.items()),  # Convert to list of tuples for env update
        )
        _ = typer.Exit(process.returncode)
        rprint(f"✅  MCP Development Running")
        # except Exception as e:
        #     rprint(f"⚠️  [yellow]Unable to initialize solver: {e}[/yellow]")
        #     raise typer.Exit(code=1)

    _ = app.command(name="dev")(mcp_development)
    return mcp_development



from pathlib import Path

from ow.workspace.model import Workspace
from ow.workspace.utils import get_project_root


def create_workspace(
    name: str,
    out_path: Path | None = None,
    force: bool = False,
    **kwargs,
) -> Workspace:
    """
    Create Workspace class object and folder.
    """

    # Use the out_path if provided, otherwise default to package out_path.
    if out_path is None:
        out_path = get_project_root() / "out"

    # Create the `out` directory if it doesn't exist.
    out_path.mkdir(exist_ok=True)

    workspace_path = out_path / name

    if workspace_path.exists() and not force:
        raise FileExistsError("Workspace already exists")

    workspace = Workspace(name=name, out_path=out_path, **kwargs)
    workspace.save()

    return workspace


def create_workspace_subfolder(
    name: str,
    subfolder: str,
    out_path: Path | None = None,
    force: bool = False,
    **kwargs,
) -> Workspace:
    """
    Create a subfolder within a Workspace and register it in workspace.json.
    """

    # Use the out_path if provided, otherwise default to package out_path.
    if out_path is None:
        out_path = get_project_root() / "out"

    if not out_path.exists() or not out_path.is_dir():
        raise FileNotFoundError

    workspace_dict_path = out_path / name / "workspace.json"

    if not workspace_dict_path.exists():
        raise FileNotFoundError

    subfolder_path = out_path / name / subfolder

    if subfolder_path.exists() and not force:
        raise FileExistsError("Workspace subfolder already exists")

    workspace = Workspace.load(workspace_dict_path)
    workspace.add_subfolder(subfolder)

    return workspace



from .initialize import register_workspace_initialize
from .resources import register_workspace_resources
from .tools import register_workspace_tools

__all__ = [
    "register_workspace_initialize",
    "register_workspace_resources",
    "register_workspace_tools",
]



import shutil
import subprocess

from importlib.resources import files
from pathlib import Path
from rich import print as rprint

from ow import data


def install(path: Path, client: str, include_agent: bool = True) -> None:
    match client:
        case "claude-code":
            # TODO: Handle case if agent already exists
            # (i.e. auto remove existing agent if updating.)
            try:
                claude_cmd = [
                    "claude",
                    "mcp",
                    "add-json",
                    "ow",
                    f'{{"command": "uv", "args": ["--directory", "{path}", "run", "-m", "ow.mcp"]}}',
                ]

                rprint(f"[blue]Running command:[/blue] {' '.join(claude_cmd)}")
                subprocess.run(claude_cmd, check=True)

                if include_agent:
                    # Copies premade agent configuration to `.claude/agents`
                    agent_file = files(data) / "mcp" / "agent.md"
                    claude_agents_path = path / ".claude" / "agents"
                    claude_agents_path.mkdir(parents=True, exist_ok=True)
                    claude_agent_config_path = claude_agents_path / "ow.md"
                    with (
                        agent_file.open("rb") as src,
                        open(claude_agent_config_path, "wb") as dst,
                    ):
                        shutil.copyfileobj(src, dst)
                    rprint(
                        f"[bold green]Installed agent under path:[/bold green] {claude_agent_config_path}"
                    )

            except subprocess.CalledProcessError as e:
                rprint(f"[red]Command failed with return code {e.returncode}[/red]")
                rprint(f"[red]Error output: {e.stderr}[/red]" if e.stderr else "")
            except Exception as e:
                rprint(f"[red]Unexpected error running command:[/red] {e}")

        case _:
            rprint(
                "[yellow]No client provided.[/yellow]\n"
                "[bold]Please specify where to install with one of the following:[/bold]\n"
                "  • [green]--client claude-code[/green] to install for Claude Code\n"
                "  • Other options coming soon..."
            )



import re

from pathlib import Path
from pydantic import BaseModel, field_validator, model_validator

from .utils import get_project_root


class Workspace(BaseModel):
    """
    Metadata for workspace.
    """

    name: str
    out_path: Path | None = None
    workspace_path: Path | None = None
    # subfolders: list[str] = []
    config_file: str = "workspace.json"

    @field_validator("name", mode="before")
    @classmethod
    def normalize_and_sanitize_name(cls, v: str) -> str:
        v = v.replace(" ", "_")
        v = re.sub(r'[<>:"/\\|?*\x00-\x1F]', "", v)
        return v[:255]

    @model_validator(mode="after")
    def populate_missing_paths(self) -> "Workspace":
        if not self.out_path:
            self.out_path = get_project_root() / "out"

        if not self.workspace_path:
            self.workspace_path = self.out_path / self.name

        return self

    # def create_subfolders(self) -> list[Path]:
    #     """
    #     Create subfolders defined in self.subfolders under workspace_path.
    #     Returns a list of created Path objects.
    #     """
    #     if not self.workspace_path:
    #         raise ValueError("workspace_path must be set before creating subfolders.")
    #
    #     created = []
    #     for folder in self.subfolders:
    #         folder_path = self.workspace_path / folder
    #         folder_path.mkdir(parents=True, exist_ok=True)
    #         created.append(folder_path)
    #
    #     self.save()
    #     return created

    # def add_subfolder(self, subfolder_name: str) -> Path:
    #     """
    #     Add and create a new subfolder inside the workspace.
    #
    #     Args:
    #         subfolder_name (str): Name of the subfolder to create.
    #
    #     Returns:
    #         Path: The path of the created subfolder.
    #
    #     Raises:
    #         ValueError: If the subfolder name contains path separators (nested paths not allowed).
    #     """
    #     if not self.workspace_path:
    #         raise ValueError("workspace_path must be set before adding subfolders.")
    #
    #     # Check for nested paths (path separators)
    #     if "/" in subfolder_name or "\\" in subfolder_name:
    #         raise ValueError(
    #             "Nested subfolder paths are not allowed. Use simple folder names only."
    #         )
    #
    #     safe_name = self.normalize_and_sanitize_name(subfolder_name)
    #     folder_path = self.workspace_path / safe_name
    #     folder_path.mkdir(parents=True, exist_ok=True)
    #
    #     if safe_name not in self.subfolders:
    #         self.subfolders.append(safe_name)
    #
    #     self.save()
    #     return folder_path

    def save(self, path: Path | None = None) -> Path:
        """
        Save the configuration to a YAML file.
        If no path is given, saves to '<workspace_path>/workspace.json'.
        """
        if path is None:
            if not self.workspace_path:
                raise ValueError(
                    "workspace_path must be set to determine save location."
                )
            path = self.workspace_path / self.config_file

        path.parent.mkdir(parents=True, exist_ok=True)
        _ = path.write_text(self.model_dump_json(indent=2))

        return path

    @classmethod
    def load(cls: type["Workspace"], path: Path) -> "Workspace":
        if not path.exists():
            raise FileNotFoundError(f"Workspace file not found at {path}")

        return cls.model_validate_json(path.read_text())



import importlib.util

from pathlib import Path


def get_project_root(parents_index: int = 4) -> Path:
    """Find project root based on package installation location."""
    try:
        spec = importlib.util.find_spec("ow")
        if spec and spec.origin:
            package_path = Path(spec.origin).parent
            parent_folder = package_path.parent.name
            if parent_folder == "src":
                # Local Development
                # package_path: /.../out-workspace/src/ow
                # package_path.parent.parent: /.../out-workspace
                return package_path.parent.parent
            else:
                # PyPI Install
                # package_path: /.../out-workspace-agent/.venv/lib/python3.13/site-packages/ow
                # package_path.parents[parents_index]: /.../out-workspace-agent
                return package_path.parents[parents_index]
    except ImportError:
        pass
    return Path.cwd()



import typer

from pathlib import Path
from rich import print as rprint

from .options import WorkspaceOption


def get_workspace_path(
    workspace: WorkspaceOption, config_file: str = "workspace.json"
) -> Path:
    """
    Checks for workspace config file in current directory or throws error.
    """
    if workspace is not None:
        # Get workspace path from name.
        from ow.workspace.utils import get_project_root

        project_root = get_project_root()
        workspace_dir = project_root / "out" / workspace

    else:
        # Check for workspace config file in current directory
        workspace_dir = Path.cwd()

    workspace_config_path = workspace_dir / config_file

    if not workspace_config_path.exists():
        rprint(
            f"❌ [red]This is not a valid workspace folder. `{workspace_config_path}` not found.[/red]"
        )
        raise typer.Exit(code=1)

    return workspace_dir



from mcp.server import FastMCP


def register_workspace_tools(app: FastMCP):
    from ow.mcp.types import ToolSuccess
    from ow.mcp.utils import tool_success
    from ow.workspace.model import Workspace

    @app.tool(
        title="List Workspaces",
        description="Provides a list of created workspaces.",
        structured_output=True,
    )
    def workspaces() -> ToolSuccess[list[str] | None]:
        from ow.workspace.tools.list import list_workspaces

        return tool_success(list_workspaces())

    @app.tool(
        title="List Workspace Subfolders",
        description="Lists of registered subfolders in a given workspace.",
        structured_output=True,
    )
    def workspace_subfolders(workspace_name: str) -> ToolSuccess[list[str] | None]:
        from ow.workspace.tools.list import list_workspace_subfolders

        return tool_success(list_workspace_subfolders(workspace_name))

    @app.tool(
        title="List Workspace Subfolder Content",
        description="Lists content within a registered subfolder for a workspace.",
        structured_output=True,
    )
    def workspace_subfolder_content(
        workspace_name: str, subfolder_name: str
    ) -> ToolSuccess[list[str] | None]:
        from ow.workspace.tools.list import list_workspace_subfolder_content

        return tool_success(
            list_workspace_subfolder_content(workspace_name, subfolder_name)
        )

    _ = (workspaces, workspace_subfolders, workspace_subfolder_content)

    @app.tool(
        title="Create Workspace Subfolder",
        description="Creates and registers a subfolder for a specific workspace",
        structured_output=True,
    )
    def workspace_subfolder_create(
        workspace_name: str, subfolder_name: str, force: bool = False
    ) -> ToolSuccess[Workspace | None]:
        from ow.workspace.tools.create import create_workspace_subfolder

        return tool_success(
            create_workspace_subfolder(workspace_name, subfolder_name, force=force)
        )

    _ = (
        workspaces,
        workspace_subfolders,
        workspace_subfolder_content,
        workspace_subfolder_create,
    )



from mcp.server import FastMCP


def register_workspace_resources(app: FastMCP):

    @app.resource("workspace://")
    def workspaces() -> list[str] | None:
        from ow.workspace.tools.list import list_workspaces

        return list_workspaces()

    @app.resource("workspace://{workspace}/")
    def workspace_subfolders(workspace: str) -> list[str] | None:
        from ow.workspace.tools.list import list_workspace_subfolders

        return list_workspace_subfolders(workspace)

    @app.resource("workspace://{workspace}/{subfolder}/")
    def workspace_subfolder_content(workspace: str, subfolder: str) -> list[str] | None:
        from ow.workspace.tools.list import list_workspace_subfolder_content

        return list_workspace_subfolder_content(workspace, subfolder)

    _ = (workspaces, workspace_subfolders, workspace_subfolder_content)



from .__main__ import app
from .options import WorkspaceOption
from .version import register_version

from ow.mcp.cli import app as mcp_app
from ow.workspace.cli import app as workspace_app

__all__ = ["app", "WorkspaceOption"]

app.add_typer(mcp_app, name="mcp")
app.add_typer(workspace_app, name="workspace")

_ = register_version(app)

if __name__ == "__main__":
    app()



import os

from pathlib import Path

from ow.workspace.utils import get_project_root


def list_workspaces(out_path: Path | None = None) -> list[str] | None:
    """
    Lists workspace directories within out_path
    """
    if out_path is None:
        out_path = get_project_root() / "out"

    if not out_path.exists():
        os.makedirs(out_path)

    if not out_path.is_dir():
        raise FileNotFoundError

    return [
        workspace_dir.name
        for workspace_dir in out_path.iterdir()
        if workspace_dir.is_dir()
    ]


def list_workspace_subfolders(
    name: str, out_path: Path | None = None
) -> list[str] | None:
    """
    Lists subfolders within a workspace.
    """
    if out_path is None:
        out_path = get_project_root() / "out"

    if not out_path.exists() or not out_path.is_dir():
        raise FileNotFoundError

    workspace_path = out_path / name

    if not workspace_path.exists() or not out_path.is_dir():
        raise FileNotFoundError

    return [
        subfolder.name for subfolder in workspace_path.iterdir() if subfolder.is_dir()
    ]


def list_workspace_subfolder_content(
    name: str, subfolder: str, out_path: Path | None = None
) -> list[str] | None:
    """
    Lists the contents within a workspace subfolder.
    """

    if out_path is None:
        out_path = get_project_root() / "out"

    if not out_path.exists() or not out_path.is_dir():
        raise FileNotFoundError

    subfolder_path = out_path / name / subfolder

    return [content.name for content in subfolder_path.iterdir()]



from .__main__ import app
from .initialize import register_workspace_initialize

# from .list import register_workspace_list

_ = register_workspace_initialize(app)
# _ = register_workspace_list(app)

__all__ = ["app"]



# import typer
#
# from pathlib import Path
# from rich import print as rprint
# from typing_extensions import Annotated
# from ow.cli import WorkspaceOption
#
#
# def print_list(name: str, values: list[str] | None = None):
#     rprint(f"\n  {name}:")
#     if values is None:
#         rprint(f"  ⚠️  [yellow]No {name} found.[/yellow]")
#     else:
#         for index, value in enumerate(values):
#             rprint(f"  {index + 1}. [cyan]{value}[/cyan]")
#
#
# def register_workspace_list(app: typer.Typer):
#     @app.command(name="list")
#     def workspace_list(
#         # meshes: Annotated[bool | None, typer.Option("--meshes", "-m", help="List meshes in workspace")] = None,
#         # parts: Annotated[bool | None, typer.Option("--parts", "-p", help="List parts in workspace")] = None,
#         # segments: Annotated[bool | None, typer.Option("--segments", "-s", help="List segments in workspace")] = None,
#         all: Annotated[
#             bool | None, typer.Option("--all", "-a", help="List all in workspace")
#         ] = None,
#         workspace: WorkspaceOption = None,
#     ) -> None:
#         """List created workspaces."""
#         if workspace is not None:
#             # Get workspace path from name.
#             from ow.workspace import WorkspaceConfig
#
#             project_root = WorkspaceConfig.get_project_root_from_package()
#             workspace_dir = project_root / "out" / workspace
#
#         else:
#             # Check for workspace config file in current directory
#             workspace_dir = Path.cwd()
#
#         config_file = workspace_dir / "workspace.json"
#
#         if not config_file.exists():
#             # Default to showing available workspaces if no workspace_path.
#             from ow.workspace.list import list_workspaces
#
#             try:
#                 workspace_names = list_workspaces()
#                 print_list("Workspaces", workspace_names)
#             except:
#                 rprint("⚠️  [yellow]Unable to list workspaces[/yellow]")
#                 _ = typer.Exit()
#         else:
#             rprint(f"Workspace: {workspace}")
#
#             # if all or meshes:
#             #     from ow.workspace.list import list_workspace_meshes
#             #
#             #     try:
#             #         workspace_meshes = list_workspace_meshes(workspace_dir)
#             #         print_list("Meshes", workspace_meshes)
#             #     except:
#             #         rprint("⚠️  [yellow]Unable to list meshes[/yellow]")
#             #         _ = typer.Exit()
#             #
#             # if all or parts:
#             #     from am.workspace.list import list_workspace_parts
#             #
#             #     try:
#             #         workspace_parts = list_workspace_parts(workspace_dir)
#             #         print_list("Parts", workspace_parts)
#             #     except:
#             #         rprint("⚠️  [yellow]Unable to list parts[/yellow]")
#             #         _ = typer.Exit()
#             #
#             # if all or segments:
#             #     from am.workspace.list import list_workspace_segments
#             #
#             #     try:
#             #         workspace_segments = list_workspace_segments(workspace_dir)
#             #         print_list("Segments", workspace_segments)
#             #     except:
#             #         rprint("⚠️  [yellow]Unable to list segments[/yellow]")
#             #         _ = typer.Exit()
#
#     _ = app.command(name="ls")(workspace_list)
#
#     _ = workspace_list
